require "lunit"

require "enl"

module("enl_parse", package.seeall, lunit.testcase)

function testParseVariable()
  str = enl.parse("<% local value = 100 %>mov %0,<%= value %>")
  assert_equal([==[local value = 100;nscr.execute('mov %0,'..value..'')]==], str)
end

function testParseMultipleCommand()
  str = enl.parse('mov %0,100:mov $0,"hoge"')
  assert_equal([==[nscr.execute('mov %0,100');nscr.execute('mov $0,"hoge"')]==], str)
end

function testParseMultipleLine()
  str = enl.parse('mov %0,100\nmov $0,"hoge"')
  assert_equal([==[
nscr.execute('mov %0,100')
nscr.execute('mov $0,"hoge"')]==], str)
end

function testParseMultipleVar()
  str = enl.parse('<% local value = 100, str = "hoge" %>mov %0,<%= value %>:mov $0,"<%= str %>"')
  assert_equal([==[local value = 100, str = "hoge";nscr.execute('mov %0,'..value..'');nscr.execute('mov $0,"'..str..'"')]==], str)
end

function testParseMultipleLineVar()
  str = enl.parse('<% local value = 100, str = "hoge" %>\nmov %0,<%= value %>\nmov $0,"<%= str %>"')
  assert_equal([==[
local value = 100, str = "hoge"
nscr.execute('mov %0,'..value..'')
nscr.execute('mov $0,"'..str..'"')]==], str)
end

function testEvalEmbeded()
  str = enl.parse([[
<% local int = 100 %>
<% local str = "hoge" %>
<% local text = "ق" %>
mov %0,<%= int %>:mov $0,"<%= str %>"
<%= text %>]])
  assert_equal([==[
local int = 100
local str = "hoge"
local text = "ق"
nscr.execute('mov %0,'..int..'');nscr.execute('mov $0,"'..str..'"')
nscr.execute(text)]==], str)
end


module("enl_eval", package.seeall, lunit.testcase)

function setup()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function teardown()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function testEvalSingleCommand()
  enl.eval('mov %0,100')
  assert_equal(100, NSGetIntValue(0))
end

function testEvalMultipleCommand()
  enl.eval('mov %0,100:mov $0,"hoge"')
  assert_equal(100, NSGetIntValue(0))
  assert_equal("hoge", NSGetStrValue(0))
end

function testEvalMultipleLine()
  enl.eval('mov %0,100\nmov $0,"hoge"')
  assert_equal(100, NSGetIntValue(0))
  assert_equal("hoge", NSGetStrValue(0))
end

function testEvalEmbeded()
  enl.eval([[
<% local int = 100 %>
<% local str = "hoge" %>
<% local text = "ق" %>
mov %0,<%= int %>:mov $0,"<%= str %>"
<%= text %>]])
  assert_equal(100, NSGetIntValue(0))
  assert_equal("hoge", NSGetStrValue(0))
  assert_equal("ق", gettext())
end


module("enl_load", package.seeall, lunit.testcase)

function setup()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function teardown()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function testLoadSimpleScript()
  enl.load("data/simple.nsc")
  assert_equal(100, NSGetIntValue(0))
  assert_equal("ق", NSGetStrValue(0))
  assert_equal([[\
͂ΎsB
NbN҂@
y[W҂\]], gettext())
end

function testLoadComplexScript()
  _G.int = 100
  _G.str = "hoge"
  _G.text = "ق"
  enl.load("data/complex.nsc")
  assert_equal(100, NSGetIntValue(0))
  assert_equal("hoge", NSGetStrValue(0))
  assert_equal([[\
͂ΎsB
NbN҂@
y[W҂\
ق]], gettext())
  _G.int = nil
  _G.str = nil
  _G.text = nil
end

function testLoadControlScript()
  enl.load("data/control.nsc")
  assert_equal(5, NSGetIntValue(0))
  local var1 = NSGetStrValue(1)
  assert_not_nil(var1:match("a=1"))
  assert_not_nil(var1:match("b=2"))
  assert_not_nil(var1:match("c=3"))
  assert_equal(0, NSGetIntValue(2))
  assert_equal(1, NSGetIntValue(3))
end

function testLoadExampleScript()
  enl.load("data/example.enl")
  assert_equal(5, NSGetIntValue(0))
  assert_equal(100, NSGetIntValue(1))
  assert_equal([[
\ĂB
1:[
2:
3:]], gettext())
end
