
require "lunit"

require "nscr"

module("nscr_alias", package.seeall, lunit.testcase)

function testAlias2Int()
  assert_equal(100, nscr.alias2int("ALIAS_INT"))
end

function testAlias2Str()
  assert_equal("hoge", nscr.alias2str("ALIAS_STR"))
end


module("nscr_execute", package.seeall, lunit.testcase)

function setup()
  NSSetIntValue(0, 0)
end

function testExecuteNormal()
  nscr.execute("mov %0,100")
  assert_equal(100, NSGetIntValue(0))
end

function testExecuteEmbeded()
  local var = 0
  nscr.execute("mov %#{var},100")
  assert_equal(100, NSGetIntValue(0))
end

function testExecuteMultiByte()
  textclear()
  nscr.execute("ق")
  assert_equal("ق", gettext())
end

function testExecuteErrorByDoubleCall()
  local f = function() nscr.execute("test_exec") end
  local result,msg = xpcall(f,function(err) return err.msg end)
  assert_false(result)
  assert_equal("NSExec2dɌĂяoƂ͂ł܂B", msg)
end


module("nscr_resolve", package.seeall, lunit.testcase)

function testResolveStr()
  local str = "hoge"
  assert_equal("hoge", nscr.resolve("#{str}"))
end

function testResolveInt()
  local int = 100
  assert_equal("100", nscr.resolve("#{int}"))
end

function testResolveBoolean()
  local bool = true
  assert_equal("true", nscr.resolve("#{bool}"))
end

function testResolveFunction()
  local func = function() return "called" end
  assert_equal("called", nscr.resolve("#{func}"))
end

function testResolveTable()
  local t = {"a", "b", "c"}
  assert_equal("abc", nscr.resolve("#{t}"))
end

function testResolveNil()
  local n = nil
  assert_equal("", nscr.resolve("#{n}"))
end

function testResolveStr()
  local str = "hoge"
  local int = 100
  local bool = true
  local func = function() return "called" end
  local t = {"a", "b", "c"}
  local n = nil
  assert_equal("head-hoge100truecalledabc-foot", nscr.resolve("head-#{str}#{int}#{true}#{func}#{t}#{n}-foot"))
end


module("nscr_pop_args", package.seeall, lunit.testcase)

function testPopInts()
  NSExec('test_pop_args "i" 1,2,3')
  assert_equal(3, #args)
  assert_equal(1, args[1])
  assert_equal(2, args[2])
  assert_equal(3, args[3])
end

function testPopStrs()
  NSExec('test_pop_args "s" "1","2","3"')
  assert_equal(3, #args)
  assert_equal("1", args[1])
  assert_equal("2", args[2])
  assert_equal("3", args[3])
end

function testPopIDs()
  NSExec('test_pop_args "#" a,b,c')
  assert_equal(3, #args)
  assert_equal("a", args[1])
  assert_equal("b", args[2])
  assert_equal("c", args[3])
end

function testPopIntRefs()
  NSExec('test_pop_args "%" %1,%2,%3')
  assert_equal(3, #args)
  assert_equal(1, args[1])
  assert_equal(2, args[2])
  assert_equal(3, args[3])
end

function testPopStrRefs()
  NSExec('test_pop_args "$" $1,$2,$3')
  assert_equal(3, #args)
  assert_equal(1, args[1])
  assert_equal(2, args[2])
  assert_equal(3, args[3])
end

function testPopAll()
  NSExec('test_pop_args "is#%$" 1,"2",c,%4,$5')
  assert_equal(5, #args)
  assert_equal(1, args[1])
  assert_equal("2", args[2])
  assert_equal("c", args[3])
  assert_equal(4, args[4])
  assert_equal(5, args[5])
end


module("nscr_getsetint", package.seeall, lunit.testcase)

function setup()
  NSSetIntValue(0, 0)
end

function teardown()
  NSSetIntValue(0, 0)
end

function testSetIntFromNumber()
  nscr.setint(0, 100)
  assert_equal(100, nscr.getint(0))
end

function testSetIntFromString()
  nscr.setint(0, "100")
  assert_equal(100, nscr.getint(0))
  nscr.setint(0, "-100")
  assert_equal(-100, nscr.getint(0))
  nscr.setint(0, "")
  assert_equal(0, nscr.getint(0))
end

function testSetIntFromBoolean()
  nscr.setint(0, true)
  assert_equal(1, nscr.getint(0))
  nscr.setint(0, false)
  assert_equal(0, nscr.getint(0))
end

function testSetIntFromFunction()
  nscr.setint(0, function() return 1 end)
  assert_equal(1, nscr.getint(0))
  nscr.setint(0, function() return "100" end)
  assert_equal(100, nscr.getint(0))
  nscr.setint(0, function() return {100,200} end)
  assert_equal(2, nscr.getint(0))
  nscr.setint(0, function() return nil end)
  assert_equal(0, nscr.getint(0))
end

function testSetIntFromTable()
  nscr.setint(0, {3})
  assert_equal(1, nscr.getint(0))
  nscr.setint(0, {n = 5})
  assert_equal(0, nscr.getint(0))
  nscr.setint(0, {})
  assert_equal(0, nscr.getint(0))
end

function testSetIntFromNil()
  nscr.setint(0, nil)
  assert_equal(0, nscr.getint(0))
end


module("nscr_getsetstr", package.seeall, lunit.testcase)

function setup()
  NSSetStrValue(0, "")
end

function teardown()
  NSSetStrValue(0, "")
end

function testSetStrFromNumber()
  nscr.setstr(0, 100)
  assert_equal("100", nscr.getstr(0))
  nscr.setstr(0, -100)
  assert_equal("-100", nscr.getstr(0))
end

function testSetStrFromString()
  nscr.setstr(0, "")
  assert_equal("", nscr.getstr(0))
end

function testSetStrFromBoolean()
  nscr.setstr(0, true)
  assert_equal("true", nscr.getstr(0))
  nscr.setstr(0, false)
  assert_equal("false", nscr.getstr(0))
end

function testSetStrFromFunction()
  nscr.setstr(0, function() return 1 end)
  assert_equal("1", nscr.getstr(0))
  nscr.setstr(0, function() return "100" end)
  assert_equal("100", nscr.getstr(0))
  nscr.setstr(0, function() return {100,200} end)
  assert_equal("100200", nscr.getstr(0))
  nscr.setstr(0, function() return nil end)
  assert_equal("", nscr.getstr(0))
end

function testSetStrFromTable()
  nscr.setstr(0, {3,5})
  assert_equal("35", nscr.getstr(0))
  nscr.setstr(0, {n = 5})
  assert_equal("", nscr.getstr(0))
  nscr.setstr(0, {})
  assert_equal("", nscr.getstr(0))
end

function testSetStrFromNil()
  nscr.setstr(0, nil)
  assert_equal("", nscr.getstr(0))
end


module("nscr_parse", package.seeall, lunit.testcase)

function testParseSingleCommand()
  local result = nscr.parse([[mov %0,100]])
  assert_equal([[nscr.execute('mov %0,100')]], result)
end

function testParseMultiCommand()
  local result = nscr.parse([[mov %0,100:mov $0,"hoge"]])
  assert_equal([[nscr.execute('mov %0,100');nscr.execute('mov $0,"hoge"')]], result)
end

function testParseMultiLine()
  local result = nscr.parse([[
mov %0,100
mov $0,"hoge"]])
  assert_equal([[
nscr.execute('mov %0,100')
nscr.execute('mov $0,"hoge"')]], result)
end

function testParseText()
  local result = nscr.parse("̐킢͂ꂩ炾I")
  assert_equal([[nscr.execute('̐킢͂ꂩ炾I')]], result)
end

function testParseStringLiteral()
  local result = nscr.parse([[mov $0,":;"]])
  assert_equal([[nscr.execute('mov $0,":;"')]], result)
end

function testParseComment()
  local result = nscr.parse([[mov %0,100 ;mov $0,":;"]])
  assert_equal([[nscr.execute('mov %0,100 ')]], result)
end


module("nscr_eval", package.seeall, lunit.testcase)

function setup()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function teardown()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function testEvalSingleCommand()
  nscr.eval([[mov %0,100]])
  assert_equal(100, NSGetIntValue(0))
end

function testEvalMultiCommand()
  nscr.eval([[mov %0,100:mov $0,"hoge"]])
  assert_equal(100, NSGetIntValue(0))
  assert_equal("hoge", NSGetStrValue(0))
end

function testEvalMultiLine()
  nscr.eval([[
mov %0,100
mov $0,"hoge"]])
  assert_equal(100, NSGetIntValue(0))
  assert_equal("hoge", NSGetStrValue(0))
end

function testEvalText()
  local text = [[\]]
  nscr.eval(text)
  assert_equal([[\]], gettext())
end

function testEvalSpecialChar()
  nscr.eval([[NbN҂@y[W҂\
NbN҂
@
y[W҂
\]])
  assert_equal([[NbN҂@y[W҂\
NbN҂
@
y[W҂
\]], gettext())
end

function testEvalStringLiteral()
  nscr.eval([[mov $0,":;"]])
  assert_equal(":;", NSGetStrValue(0))
end

function testEvalComment()
  nscr.eval([[mov %0,100 ;mov $0,":;"]])
  assert_equal(100, NSGetIntValue(0))
  assert_equal("", NSGetStrValue(0))
end

module("nscr_load", package.seeall, lunit.testcase)

function setup()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function teardown()
  NSSetIntValue(0, 0)
  NSSetStrValue(0, "")
  textclear()
end

function testLoadSimpleScript()
  nscr.load("data/simple.nsc")
  assert_equal(100, NSGetIntValue(0))
  assert_equal("ق", NSGetStrValue(0))
  assert_equal([[\
͂ΎsB
NbN҂@
y[W҂\]], gettext())
end


module("nscr_mement", package.seeall, lunit.testcase)

function setup()
  NSSetIntValue(0, 100)
  NSSetIntValue(1, 110)
  NSSetIntValue(2, 120)
  NSSetIntValue(3, 130)
  NSSetStrValue(0, "hoge")
  NSSetStrValue(1, "foo")
  NSSetStrValue(2, "bar")
  NSSetStrValue(3, "buz")
end

function teardown()
  NSSetIntValue(0, 0)
  NSSetIntValue(1, 0)
  NSSetIntValue(2, 0)
  NSSetIntValue(3, 0)
  NSSetStrValue(0, "")
  NSSetStrValue(1, "")
  NSSetStrValue(2, "")
  NSSetStrValue(3, "")
end

function testMementInt()
  nscr.mementint(0)
  NSSetIntValue(0, 5)
  NSSetStrValue(0, "fuga")
  nscr.rememberint(0)
  assert_equal(100, NSGetIntValue(0))
  assert_equal("fuga", NSGetStrValue(0)) -- not remembered
end

function testMementStr()
  nscr.mementstr(0)
  NSSetIntValue(0, 5)
  NSSetStrValue(0, "fuga")
  nscr.rememberstr(0)
  assert_equal(5, NSGetIntValue(0)) -- not remembered
  assert_equal("hoge", NSGetStrValue(0))
end

function testMementBothResumeInt()
  nscr.mement(0)
  NSSetIntValue(0, 5)
  NSSetStrValue(0, "fuga")
  nscr.rememberint(0)
  assert_equal(100, NSGetIntValue(0))
  assert_equal("fuga", NSGetStrValue(0)) -- not remembered
end

function testMementBothResumeStr()
  nscr.mement(0)
  NSSetIntValue(0, 5)
  NSSetStrValue(0, "fuga")
  nscr.rememberstr(0)
  assert_equal(5, NSGetIntValue(0)) -- not remembered
  assert_equal("hoge", NSGetStrValue(0))
end

function testMementBothResumeBoth()
  nscr.mement(0)
  NSSetIntValue(0, 5)
  NSSetStrValue(0, "fuga")
  nscr.remember(0)
  assert_equal(100, NSGetIntValue(0))
  assert_equal("hoge", NSGetStrValue(0))
end

function testMementRangeResumeRange()
  nscr.mement(0, 4)
  NSSetIntValue(0, 0)
  NSSetIntValue(1, 1)
  NSSetIntValue(2, 2)
  NSSetIntValue(3, 3)
  NSSetStrValue(0, "a")
  NSSetStrValue(1, "b")
  NSSetStrValue(2, "c")
  NSSetStrValue(3, "d")
  nscr.remember(0, 4)
  assert_equal(100, NSGetIntValue(0))
  assert_equal(110, NSGetIntValue(1))
  assert_equal(120, NSGetIntValue(2))
  assert_equal(130, NSGetIntValue(3))
  assert_equal("hoge", NSGetStrValue(0))
  assert_equal("foo", NSGetStrValue(1))
  assert_equal("bar", NSGetStrValue(2))
  assert_equal("buz", NSGetStrValue(3))
end

function testMementRangeResumeAll()
  nscr.mement(0, 4)
  NSSetIntValue(0, 0)
  NSSetIntValue(1, 1)
  NSSetIntValue(2, 2)
  NSSetIntValue(3, 3)
  NSSetStrValue(0, "a")
  NSSetStrValue(1, "b")
  NSSetStrValue(2, "c")
  NSSetStrValue(3, "d")
  nscr.remember()
  assert_equal(100, NSGetIntValue(0))
  assert_equal(110, NSGetIntValue(1))
  assert_equal(120, NSGetIntValue(2))
  assert_equal(130, NSGetIntValue(3))
  assert_equal("hoge", NSGetStrValue(0))
  assert_equal("foo", NSGetStrValue(1))
  assert_equal("bar", NSGetStrValue(2))
  assert_equal("buz", NSGetStrValue(3))
end
